# Generated by Django 5.2.9 on 2026-01-09 14:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wbcore', '0014_biguserobjectpermission_system'),
    ]

    operations = [
        migrations.DeleteModel(
            name='GenericModel',
        ),
    ]
