# Generated by Django 5.2.9 on 2026-01-26 12:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0007_alter_token_unique_together_token_unique_token'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='is_internal',
            field=models.BooleanField(default=False, help_text='If true, define this user as internal which grants specific permissions.', verbose_name='Internal'),
        ),
    ]
